/*
 * Copyright 2017, Cypress Semiconductor Corporation or a subsidiary of 
 * Cypress Semiconductor Corporation. All Rights Reserved.
 * 
 * This software, associated documentation and materials ("Software"),
 * is owned by Cypress Semiconductor Corporation
 * or one of its subsidiaries ("Cypress") and is protected by and subject to
 * worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions.
 * Therefore, you may use this Software only as provided in the license
 * agreement accompanying the software package from which you
 * obtained this Software ("EULA").
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
 * non-transferable license to copy, modify, and compile the Software
 * source code solely for use in connection with Cypress's
 * integrated circuit products. Any reproduction, modification, translation,
 * compilation, or representation of this Software except as specified
 * above is prohibited without the express written permission of Cypress.
 *
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
 * reserves the right to make changes to the Software without notice. Cypress
 * does not assume any liability arising out of the application or use of the
 * Software or any product or circuit described in the Software. Cypress does
 * not authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death ("High Risk Product"). By
 * including Cypress's product in a High Risk Product, the manufacturer
 * of such system or application assumes all risk of such use and in doing
 * so agrees to indemnify Cypress against all liability.
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "wiced.h"

#include "wiced_rtos.h"

#include "wiced_bt_stack.h"
#include "wiced_bt_dev.h"
#include "wiced_bt_ble.h"
#include "wiced_bt_gatt.h"
#include "wiced_bt_cfg.h"

/******************************************************
 *                      Macros
 ******************************************************/

/******************************************************
 *                    Constants
 ******************************************************/


/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Static Function Declarations
 ******************************************************/

/******************************************************
 *               Variable Definitions
 ******************************************************/

extern wiced_bt_cfg_settings_t         ble_wiced_bt_cfg_settings;
extern const wiced_bt_cfg_buf_pool_t   ble_wiced_bt_cfg_buf_pools[];

/******************************************************
 *               Function Declarations
 ******************************************************/
static wiced_bt_dev_status_t  bt_stack_init_callback   ( wiced_bt_management_evt_t event, wiced_bt_management_evt_data_t *p_event_data );
static wiced_bt_gatt_status_t bt_connect_gatt_event_callback( wiced_bt_gatt_evt_t event, wiced_bt_gatt_event_data_t* p_event_data );
static void bt_scan_result_callback( wiced_bt_ble_scan_results_t *p_scan_result, uint8_t *raw_data );

static wiced_bt_device_address_t       test_device_address;
static uint8_t                         ble_address_type;
static wiced_bt_gatt_discovery_param_t ble_parameters;

int 					try_counter=0;
uint16_t				connection_id;

/******************************************************
 *               Function Definitions
 ******************************************************/

static wiced_bt_gatt_status_t bt_connect_gatt_event_callback( wiced_bt_gatt_evt_t event, wiced_bt_gatt_event_data_t* p_event_data )
{
    wiced_bt_gatt_status_t    status = WICED_BT_GATT_SUCCESS;
	
    switch ( event )
    {
        case GATT_CONNECTION_STATUS_EVT:
            if ( p_event_data->connection_status.connected == WICED_TRUE )
            {
	printf("Connect to [%02x:%02x:%02x:%02x:%02x:%02x] conn_id=%d\r\n",
			p_event_data->connection_status.bd_addr[0], p_event_data->connection_status.bd_addr[1], p_event_data->connection_status.bd_addr[2], p_event_data->connection_status.bd_addr[3], p_event_data->connection_status.bd_addr[4],p_event_data->connection_status.bd_addr[5],
			p_event_data->connection_status.conn_id);

				connection_id = p_event_data->connection_status.conn_id;

                /* Discover HPS service and its start and end handles */
                memset( &ble_parameters, 0, sizeof(wiced_bt_gatt_discovery_param_t) );

                ble_parameters.s_handle       = 1;
                ble_parameters.e_handle       = 0xFFFF;
                wiced_bt_gatt_send_discover( connection_id, GATT_DISCOVER_SERVICES_ALL, &ble_parameters );
			}
            else
            {
	printf("Disconnect to [%02x:%02x:%02x:%02x:%02x:%02x] reason=0x%x\r\n", 
			p_event_data->connection_status.bd_addr[0], p_event_data->connection_status.bd_addr[1], p_event_data->connection_status.bd_addr[2], p_event_data->connection_status.bd_addr[3], p_event_data->connection_status.bd_addr[4],p_event_data->connection_status.bd_addr[5],
			p_event_data->connection_status.reason);
				
				connection_id = 0;

				//wait 1 seconds
				wiced_rtos_delay_milliseconds(1000);
				//restart scan
			    if(wiced_bt_ble_scan( BTM_BLE_SCAN_TYPE_HIGH_DUTY, WICED_TRUE, bt_scan_result_callback) != WICED_BT_PENDING) 
				{
			        printf("wiced_bt_ble_scan restart fail!!!\r\n");
			    }
				printf("\r\n\r\n");
				printf("wiced_bt_ble_scan restart success!!!\r\n");
			}

            break;

        case GATT_DISCOVERY_RESULT_EVT:
            break;

        case GATT_DISCOVERY_CPLT_EVT:

				printf("call wiced_bt_gatt_disconnect connection_id=%d\r\n", connection_id);
                wiced_bt_gatt_disconnect( connection_id );
            break;

        default:
            printf("Unhandled telehealth_connect_gatt_event_callback Event: 0x%x\n", event);
            break;
    }

    return status;
}

static void bt_scan_result_callback( wiced_bt_ble_scan_results_t *p_scan_result, uint8_t *raw_data )
{
    if ( p_scan_result != NULL )
	{
		if( (p_scan_result->remote_bd_addr[ 0 ] == 0x30) &&
			(p_scan_result->remote_bd_addr[ 1 ] == 0xe0) &&
			(p_scan_result->remote_bd_addr[ 2 ] == 0x90) &&
			(p_scan_result->remote_bd_addr[ 3 ] == 0x15) &&
			(p_scan_result->remote_bd_addr[ 4 ] == 0x01) &&
			(p_scan_result->remote_bd_addr[ 5 ] == 0x06))
		{
			/* Copy the address */
			memcpy( test_device_address, p_scan_result->remote_bd_addr, BD_ADDR_LEN );
			ble_address_type = p_scan_result->ble_addr_type;

			printf("wiced_bt_ble_scan stop scan!!!\r\n");
			wiced_bt_ble_scan( BTM_BLE_SCAN_TYPE_NONE, WICED_TRUE, NULL );
		}
    }
    else /* Scan completion event occurs when the callback gets called with p_scan_result set to NULL */
    {
        if ( ( test_device_address[ 0 ] | test_device_address[ 1 ] | test_device_address[ 2 ] |
               test_device_address[ 3 ] | test_device_address[ 4 ] | test_device_address[ 5 ] ) != 0 )
        {
 			try_counter++;
			printf("try to connect [%02x:%02x:%02x:%02x:%02x:%02x] try_counter=%d\r\n",
					test_device_address[0], test_device_address[1], test_device_address[2], test_device_address[3], test_device_address[4],test_device_address[5], try_counter);
		    if(wiced_bt_gatt_le_connect(test_device_address, ble_address_type, BLE_CONN_MODE_HIGH_DUTY, WICED_TRUE) != WICED_TRUE) 
			{
		        printf("wiced_bt_gatt_le_connect fail!!!\r\n");
		    }
        }
   }


}

static wiced_bt_dev_status_t bt_stack_init_callback( wiced_bt_management_evt_t event, wiced_bt_management_evt_data_t* p_event_data )
{
    wiced_bt_dev_status_t       status = WICED_BT_SUCCESS;

    switch ( event )
    {
        case BTM_ENABLED_EVT:
            /* Initialize GATT REST API Server once Bluetooth controller and host stack is enabled */
            if ( ( status = p_event_data->enabled.status ) != WICED_BT_SUCCESS ) {
				printf("telehealth_bt_stack_init_callback fail!!!\r\n");
                break;
            }

			wiced_bt_gatt_register( bt_connect_gatt_event_callback );

		    if(wiced_bt_ble_scan( BTM_BLE_SCAN_TYPE_HIGH_DUTY, WICED_TRUE, bt_scan_result_callback) != WICED_BT_PENDING) 
			{
		        printf("wiced_bt_ble_scan start fail!!!\r\n");
		    }
			printf("wiced_bt_ble_scan start success!!!\r\n");

            break;

        case BTM_DISABLED_EVT:
            break;

        case BTM_BLE_SCAN_STATE_CHANGED_EVT:
            break;

        default:
//            printf("Unhandled Bluetooth Management Event: 0x%x\n", event);
            break;
    }

    return status;
}

void application_start( )
{
	wiced_result_t ret;

    wiced_init( );

// print a lot dummy string to make connect success.
#if 0

printf("test1232345\r\n");
printf("testerdfgdfdfb\r\n");
printf("testjh,muylyutyty\r\n");
printf("testwergbnbtmyuiyukmym\r\n");
printf("testtry7mhjmfgndfbbd\r\n");
printf("test234eggfh,jluouio\r\n");
printf("test545hfthntyyukiykuikmuitukytu\r\n");

printf("eg454y5u67k,hgmhmmeme\r\n");
printf("432erwgrnhjykuyiko6ol68t\r\n");
printf("3gwgretyrh67j67juik899l8iyol68l\r\n");
printf("ukjykuyilpi;owrwegegeg45rtgb\r\n");
printf("egegerhtrjyjytktku,t,ut,tu,tu,\r\n");
printf("rntery4yrtytyjtjrtjtrj\r\n");
printf("4hy4rrwrhtjy7yl9l09l89lyjmng\r\n");


printf("fortestfntyhujtjyrrymyrmrymte\r\n");
printf("yukyuiourgger34556hjerfnfnfgbdbdb\r\n");
printf("wer2343265h5t6jyukuyli.tyjrkjyk,09;\r\n");
printf("4rhrhrehehtrjjj.tjtjtj;ebwrbwrtwr\r\n");
printf("fhhetyjrjrymtu,mtu,tu,yuyutu\r\n");
printf("mymuiko7k46ejwbnrbfgnfdndfndf\r\n");

#endif

    /* Initialize BT host stack and controller */
    ret = wiced_bt_stack_init( bt_stack_init_callback, &ble_wiced_bt_cfg_settings, ble_wiced_bt_cfg_buf_pools );
printf("application_start wiced_bt_stack_init ret=%d\r\n", ret);
printf("wiced_bt_cfg_settings.device_name=%s\r\n", ble_wiced_bt_cfg_settings.device_name);

}


